/* Program  ar.cc            */
/* By Harald Stauss          */
/* Last change 11. Oct. 2000 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "tsa.h"
#include "nrc/nrutil.h" 
#include "nrc/nrutil.c"
#include "nrc/moment.c"
#include "nrc/memcof.c"
#include "nrc/evlmem.c"

main(int argc, char *argv[])
{
  REGIST        r,o;
  int           i, nlags;
  unsigned long n, nout;
  double        f, fdt, tp;
  float         sr=1.0, *d, xms;
  float         ave, adev, sdev, var, skew, curt;
  char          *fin, *fout, *ftin, *ftout;
  char          *fpin, *fpout, *fptin, *fptout;

  cout << "AR Ver. 1.0 by H.M. Stauss, 2000 \n";

  /* A r r a y s   a l l o c i e r e n */
  fin=(char *)malloc(255); fpin=fin;
  fout=(char *)malloc(255); fpout=fout;
  ftin=(char *)malloc(255); fptin=ftin;
  ftout=(char *)malloc(255); fptout=ftout;
  
  if (argc!=5 && argc!=6)
    {
      cout << "Number of parameters wrong! \n";
      cout << "Calculates Autoregressive Spectra \n";
      cout << "ar fin fout nout nlags [sr]\n";
      cout << "fin    : name of input file. \n";
      cout << "fout   : name of output file. \n";
      cout << "nout   : number of output data values \n";
      cout << "nlags  : number of lags for AR-model \n";
      cout << "[sr]   : sampling rate if ASCII file \n";
      goto ende;
    }

  // Parameter einlesen;
  strcpy(fin ,argv[1]);
  strcpy(ftin,fin);
  fptin=strrchr(ftin,'.');
  for (i=0; fptin[i]; i++) fptin[i]=toupper(fptin[i]);
  strcpy(fout ,argv[2]);
  strcpy(ftout,fout);
  fptout=strrchr(ftout,'.');
  for (i=0; fptout[i]; i++) fptout[i]=toupper(fptout[i]);
  sscanf(argv[3],"%ul",&nout);
  sscanf(argv[4],"%d",&nlags);
  if (argc==6) sscanf(argv[5],"%f",&sr);
  if ((argc==5) && (!strcmp(fptin,".ASC"))) {
    cout << "Need sampling rate for ASCII files! \n";
    goto ende;
  }

  /* A r r a y s   a l l o c i e r e n */
  d=new float[nlags];

  // E i n g a b e d a t e i e n   o e f f n e n ;
  if (strcmp(fptin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(fptin,".ASC")==0) r.ReadASCII(fin,sr);
  if ((strcmp(fptin,".TSA")!=0) && (strcmp(fptin,".ASC")!=0)) {
    cout << "File type for input file not recognized! \n";
    goto ende;
  }

  cout << "In-File           : " << fin << "\n";
  cout << "Out-File          : " << fout << "\n";
  cout << "Number of lags    : " << nlags << "\n";
  cout << "Number of channels: " << r.n << "\n";
     
  // A u t o r e g r e s s i o n ;
  for (i=0; i<r.n; i++) {
    cout << i+1 << ": number of lines: " << r.ch[i].n << "\n";
    o.AddChannel(nout, r.ch[i].sr,
		 r.ch[i].min, r.ch[i].max,
		 r.ch[i].height, r.ch[i].color,
		 r.ch[i].name, r.ch[i].unit);
    // Mittelwert berechnen und abziehen;
    moment(r.ch[i].data-1, r.ch[i].n, &ave, &adev, &sdev, &var, &skew, &curt);
    cout << i+1 << ":            mean: " << ave << "\n";
    cout << i+1 << ":        variance: " << var << "\n";
    for (n=0; n<r.ch[i].n; n++) r.ch[i].data[n]-=ave;
    // Coefficients;
    memcof(r.ch[i].data-1, r.ch[i].n, nlags, &xms, d-1);
    fdt=r.ch[i].sr/(2*(nout-1));
    f=0; tp=0;
    for (n=0; n<nout; n++) {
      o.ch[i].data[n]=evlmem(f/r.ch[i].sr, d-1, nlags, xms)/(float)(nout);
      f+=fdt;
      tp+=o.ch[i].data[n];
    }
    cout << i+1 << ":     total power: " << tp << "\n";
    for (n=0; n<nout; n++) {
      if (o.ch[i].data[n] < r.ch[i].min) r.ch[i].min=o.ch[i].data[n];
      if (o.ch[i].data[n] > r.ch[i].max) r.ch[i].max=o.ch[i].data[n];
    }
  }
     
  // D a t e i   s c h r e i b e n ;
  if (strcmp(fptout,".TSA")==0) o.WriteTSA(fout);
  if (strcmp(fptout,".ASC")==0) o.WriteASCII(fout);
  if ((strcmp(fptout,".TSA")!=0) && (strcmp(fptout,".ASC")!=0)) {
    printf("File type for output not recognized! \n");
    goto ende;
  }     
     
     
 ende:
  free(ftin);free(fin);free(fout);free(ftout);   
  delete[]d;
  return(0);
} /* Ende von main */







